#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/i2c.h>
#include <zephyr/sys/printk.h>

#define I2C0_NODE DT_NODELABEL(i2c0)
#define I2C1_NODE DT_NODELABEL(i2c1)

const struct device *i2c0_dev = DEVICE_DT_GET(I2C0_NODE);
const struct device *i2c1_dev = DEVICE_DT_GET(I2C1_NODE);

void scan_i2c_bus(const struct device *i2c_dev, const char *bus_name) {
    if (!device_is_ready(i2c_dev)) {
        printk("❌ I2C %s is not ready!\n", bus_name);
        return;
    }

    printk("🔎 Scanning %s...\n", bus_name);
    for (uint8_t addr = 0x08; addr <= 0x77; addr++) {
        uint8_t dummy;
        struct i2c_msg msg = {
            .buf = &dummy,
            .len = 1,
            .flags = I2C_MSG_WRITE | I2C_MSG_STOP,
        };

        if (i2c_transfer(i2c_dev, &msg, 1, addr) == 0) {
            printk("Found device at 0x%02X on %s\n", addr, bus_name);
        }
    }
}

int main(void) {
    printk("\n🚀 Starting I2C scan on nRF52840\n");

    scan_i2c_bus(i2c0_dev, "I2C0");
    scan_i2c_bus(i2c1_dev, "I2C1");

    printk("🎉 I2C scan completed.\n");
    return 0;
}
